<?php
/**
 * A class for an HTTP response on an HTTP request
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class HttpResponse extends BaseFrameworkSystem implements Responseable {
	/**
	 * Response status
	 */
	private $responseStatus = "200 OK";

	/**
	 * Array with all headers
	 */
	private $responseHeaders = array();

	/**
	 * Body of the response
	 */
	private $responseBody = "";

	/**
	 * Instance of the template engine
	 */
	private $templateEngine = null;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set part description
		$this->setObjectDescription("HTTP-Antwort");

		// Create unique ID number
		$this->createUniqueID();

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Creates an object of this class
	 *
	 * @param	$appInstance		An instance of a manageable application
	 * @return	$responseInstance	A prepared instance of this class
	 */
	public final static function createHttpResponse (ManageableApplication $appInstance) {
		// Get a new instance
		$responseInstance = new HttpResponse();

		// Initialize the template engine here
		$responseInstance->initTemplateEngine($appInstance);

		// Return the prepared instance
		return $responseInstance;
	}

	/**
	 * Setter for status
	 *
	 * @param	$status		New response status
	 * @return	void
	 */
	public final function setResponseStatus ($status) {
		$this->responseStatus = (string) $status;
	}

	/**
	 * Add header element
	 *
	 * @param	$name	Name of header element
	 * @param	$value	Value of header element
	 * @return	void
	 */
	public final function addHeader ($name, $value) {
		$this->responseHeaders[$name] = $value;
	}

	/**
	 * "Writes" data to the response body
	 *
	 * @param	$output		Output we shall sent in the HTTP response
	 * @return	void
	 */
	public function writeToBody ($output) {
		$this->responseBody .= $output;
	}

	/**
	 * Flushs the cached HTTP response to the outer world
	 *
	 * @param	$foce	Wether we shall force the output or abort if headers are
	 *					already sent with an exception
	 * @return	void
	 * @throws	ResponseHeadersAlreadySentException		Thrown if headers are
	 *													already sent
	 */
	public function flushBuffer ($force=false) {
		if ((headers_sent()) && (!$force)) {
			// Headers are already sent!
			throw new ResponseHeadersAlreadySentException($this, self::EXCEPTION_HEADERS_ALREADY_SENT);
		} elseif (!headers_sent()) {
			// Send headers out
			header("HTTP/1.0 {$this->responseStatus}");
			foreach ($this->responseHeaders as $name=>$value) {
				header("{$name}: {$value}");
			}
		}

		// Flush the output to the world
		$this->getWebOutputInstance()->output($this->responseBody);
		$this->reponseBody = "";
		$this->responseHeaders = array();
	}

	/**
	 * Initializes the template engine instance
	 *
	 * @param	$appInstance	An instance of a manageable application
	 * @return	void
	 */
	public final function initTemplateEngine (ManageableApplication $appInstance) {
		$this->templateEngine = $this->prepareTemplateEngine($appInstance);
	}

	/**
	 * Getter for the template engine instance
	 *
	 * @return	$templateEngine		An instance of the used template engine
	 */
	public final function getTemplateEngine () {
		return $this->templateEngine;
	}
}

// [EOF]
?>
